//+------------------------------------------------------------------+
//|                                              TickSpikeHunter.mq5 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_plots   2
//--- plot TickPrice
#property indicator_label1  "TickPrice"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrGreen
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot Signal
#property indicator_label2  "Signal"
#property indicator_type2   DRAW_COLOR_ARROW
#property indicator_color2  clrRed,clrBlue,C'0,0,0',C'0,0,0',C'0,0,0',C'0,0,0',C'0,0,0',C'0,0,0'
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- input parameters
input int      ticks=50;            //    
input int      shift=200;           // -  
input double   gap=3.0;             //    
input bool     optimizeCalc=true;   //  
//--- indicator buffers
double         TickPriceBuffer[];
double         SignalBuffer[];
double         SignalColors[];
double         DeltaTickBuffer[];
//---   
int ticks_counter;
//---   
bool first;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,TickPriceBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,SignalBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,SignalColors,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(3,DeltaTickBuffer,INDICATOR_CALCULATIONS);
//---   ,       
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,0);
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,0);
//---       
   PlotIndexSetInteger(1,PLOT_ARROW,159);
//---   
   ticks_counter=0;
   first=true;
//---   
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---          
   if(first)
     {
      ZeroMemory(TickPriceBuffer);
      ZeroMemory(SignalBuffer);
      ZeroMemory(SignalColors);
      ZeroMemory(DeltaTickBuffer);
      //---     ,     
      ArraySetAsSeries(SignalBuffer,true);
      ArraySetAsSeries(TickPriceBuffer,true);
      ArraySetAsSeries(SignalColors,true);
      ArraySetAsSeries(DeltaTickBuffer,true);
      first=false;
     }
//---       Close
   double lastprice=close[rates_total-1];
//---   
   if(IS_DEBUG_MODE)
     {
      //---     Bid
      double lastpricedouble=SymbolInfoDouble(_Symbol,SYMBOL_BID);
      //---     Bid
      MqlTick tick;
      SymbolInfoTick(_Symbol,tick);
      double lastpricetick=tick.bid;
     }
//---     
   if(lastprice!=TickPriceBuffer[0])
     {
      //---  
      ticks_counter++;
      ApplyTick(lastprice); //               
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
void ApplyTick(double price)
  {
//---     TickPriceBuffer -   -   
   static int prev_size=-1;
   int size=ArraySize(TickPriceBuffer);
//---      ,      1  
   if(size==prev_size)
     {
      //---         
      int move=size-1;
      if(shift!=0) move=shift;
      ArrayCopy(TickPriceBuffer,TickPriceBuffer,1,0,move);
      ArrayCopy(SignalBuffer,SignalBuffer,1,0,move);
      ArrayCopy(SignalColors,SignalColors,1,0,move);
      ArrayCopy(DeltaTickBuffer,DeltaTickBuffer,1,0,move);
     }
   else
      prev_size=size;
//---    
   TickPriceBuffer[0]=price;
//---     
   DeltaTickBuffer[0]=TickPriceBuffer[0]-TickPriceBuffer[1];
//---  . . 
   double stddev;
   if(optimizeCalc)
      stddev=getStdDevOptimized(ticks);
   else
      stddev=getStdDev(ticks);
//---       
   if(MathAbs(DeltaTickBuffer[0])>gap*stddev) //      ,   
     {
      SignalBuffer[0]=price;     //  
      string col="Red";          //  ,   
      if(DeltaTickBuffer[0]>0)   //   
        {
         SignalColors[0]=1;      //    
         col="Blue";             //     
        }
      else                       //   
      SignalColors[0]=0;         //   
      //---     
      PrintFormat("tick=%G change=%.1f pts, trigger=%.3f pts,  stddev=%.3f pts %s",
                  TickPriceBuffer[0],DeltaTickBuffer[0]/_Point,gap*stddev/_Point,stddev/_Point,col);
     }
   else SignalBuffer[0]=0;       //     
  }
//+------------------------------------------------------------------+
//|    " "                         |
//+------------------------------------------------------------------+
double getStdDev(int number)
  {
   double summ=0,sum2=0,average,stddev;
//---      
   for(int i=0;i<ticks;i++)
      summ+=DeltaTickBuffer[i];
   average=summ/ticks;
//---    
   sum2=0;
   for(int i=0;i<ticks;i++)
      sum2+=(DeltaTickBuffer[i]-average)*(DeltaTickBuffer[i]-average);
   stddev=MathSqrt(sum2/(number-1));
//--- 
   if(IS_DEBUG_MODE)
     {
      if(ticks_counter==1)
        {
         PrintFormat("summ=%G  average=%G  sum2=%G",summ,average,sum2);
         for(int i=0;i<ticks;i++)
           {
            PrintFormat("TickPriceBuffer[%d]=%G DeltaTickBuffer[%d]=%G",
                        i,TickPriceBuffer[i],i,DeltaTickBuffer[i]);
           }
        }
     }
//---  
   if(IS_DEBUG_MODE)
     {
      if(ticks_counter==1)
        {
         PrintFormat("summ=%G  average=%G  sum2=%G",summ,average,sum2);
         for(int i=0;i<ticks;i++)
           {
            PrintFormat("TickPriceBuffer[%d]=%G DeltaTickBuffer[%d]=%G",
                        i,TickPriceBuffer[i],i,DeltaTickBuffer[i]);
           }
        }
     }
//---
   return (stddev);
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
double getStdDevOptimized(int number)
  {
//---
   static double X2[],X[],X2sum=0,Xsum=0;
   static bool firstcall=true;
//---  
   if(firstcall)
     {
      //---      1   
      ArrayResize(X2,ticks+1);
      ArrayResize(X,ticks+1);
      //---       
      ZeroMemory(X2);
      ZeroMemory(X);

      firstcall=false;
     }
//---  
   ArrayCopy(X,X,1,0,ticks);
   ArrayCopy(X2,X2,1,0,ticks);
//---     
   X[0]=DeltaTickBuffer[0];
   X2[0]=DeltaTickBuffer[0]*DeltaTickBuffer[0];
//---   
   Xsum=Xsum+X[0]-X[ticks];
   X2sum=X2sum+X2[0]-X2[ticks];
//---   
   double S2=(1.0/(ticks-1))*(X2sum-Xsum*Xsum/ticks);
//---      
   double stddev=MathSqrt(S2);
//---
   return (stddev);
  }
//+------------------------------------------------------------------+
